@@ -39,7 +39,6 @@
 #include "mountoperation.h"
 #include "outputstream.h"
 #include "seekable.h"
-#include "simpleasyncresult.h"
 #include "volume.h"
 #include "volumemonitor.h"
 #include "bufferedinputstream.h"
@@ -75,7 +74,6 @@ GType g_input_stream_get_type(void);
 GType g_memory_input_stream_get_type(void);
 GType g_mount_operation_get_type(void);
 GType g_output_stream_get_type(void);
-GType g_simple_async_result_get_type(void);
 GType g_themed_icon_get_type(void);
 GType g_unix_input_stream_get_type(void);
 GType g_unix_output_stream_get_type(void);
@@ -109,7 +107,6 @@ namespace Gio {  class InputStream_Class { public: static Glib::ObjectBase* wrap
 namespace Gio {  class MemoryInputStream_Class { public: static Glib::ObjectBase* wrap_new(GObject*); };  }
 namespace Gio {  class MountOperation_Class { public: static Glib::ObjectBase* wrap_new(GObject*); };  }
 namespace Gio {  class OutputStream_Class { public: static Glib::ObjectBase* wrap_new(GObject*); };  }
-namespace Gio {  class SimpleAsyncResult_Class { public: static Glib::ObjectBase* wrap_new(GObject*); };  }
 namespace Gio {  class ThemedIcon_Class { public: static Glib::ObjectBase* wrap_new(GObject*); };  }
 namespace Gio {  class UnixInputStream_Class { public: static Glib::ObjectBase* wrap_new(GObject*); };  }
 namespace Gio {  class UnixOutputStream_Class { public: static Glib::ObjectBase* wrap_new(GObject*); };  }
@@ -143,7 +140,6 @@ void wrap_init()
   Glib::wrap_register(g_memory_input_stream_get_type(), &Gio::MemoryInputStream_Class::wrap_new);
   Glib::wrap_register(g_mount_operation_get_type(), &Gio::MountOperation_Class::wrap_new);
   Glib::wrap_register(g_output_stream_get_type(), &Gio::OutputStream_Class::wrap_new);
-  Glib::wrap_register(g_simple_async_result_get_type(), &Gio::SimpleAsyncResult_Class::wrap_new);
   Glib::wrap_register(g_themed_icon_get_type(), &Gio::ThemedIcon_Class::wrap_new);
   Glib::wrap_register(g_unix_input_stream_get_type(), &Gio::UnixInputStream_Class::wrap_new);
   Glib::wrap_register(g_unix_output_stream_get_type(), &Gio::UnixOutputStream_Class::wrap_new);
@@ -170,7 +166,6 @@ void wrap_init()
   Gio::MemoryInputStream::get_type();
   Gio::MountOperation::get_type();
   Gio::OutputStream::get_type();
-  Gio::SimpleAsyncResult::get_type();
   Gio::ThemedIcon::get_type();
   Gio::UnixInputStream::get_type();
   Gio::UnixOutputStream::get_type();
