@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: objectbase.cc 337 2006-11-10 02:24:49Z murrayc $ */
+/* $Id: objectbase.cc 530 2008-01-20 18:54:44Z murrayc $ */
 
 /* Copyright 2002 The gtkmm Development Team
  *
@@ -257,6 +257,11 @@ void ObjectBase::get_property_value(const Glib::ustring& property_name, Glib::Va
 }
 
 void ObjectBase::connect_property_changed(const Glib::ustring& property_name, const sigc::slot<void>& slot)
+{
+  connect_property_changed_with_return(property_name, slot);
+}
+
+sigc::connection ObjectBase::connect_property_changed_with_return(const Glib::ustring& property_name, const sigc::slot<void>& slot)
 {
   // Create a proxy to hold our connection info
   // This will be deleted by destroy_notify_handler.
@@ -271,9 +276,12 @@ void ObjectBase::connect_property_changed(const Glib::ustring& property_name, co
          notify_signal_name.c_str(), (GCallback)(&PropertyProxyConnectionNode::callback), pConnectionNode, 
          &PropertyProxyConnectionNode::destroy_notify_handler,
          G_CONNECT_AFTER);
+
+  return sigc::connection(pConnectionNode->slot_);
 }
 
 
+
 bool _gobject_cppinstance_already_deleted(GObject* gobject)
 {
   //This function is used to prevent calling wrap() on a GTK+ instance whose gtkmm instance has been deleted.
