@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: value_custom.h 190 2005-01-21 19:26:04Z murrayc $ */
+/* $Id: value_custom.h 419 2007-06-22 14:43:53Z murrayc $ */
 
 /* Copyright 2002 The gtkmm Development Team
  *
@@ -18,6 +18,9 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifndef _GLIBMM_VALUE_CUSTOM_H
+#define _GLIBMM_VALUE_CUSTOM_H
+
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 #ifndef _GLIBMM_VALUE_H_INCLUDE_VALUE_CUSTOM_H
 #error "glibmm/value_custom.h cannot be included directly"
@@ -287,3 +290,6 @@ void Value<T>::value_copy_func(const GValue* src_value, GValue* dest_value)
 
 } // namespace Glib
 
+#endif //_GLIBMM_VALUE_CUSTOM_H
+
+
